'''
Author: Jiashu Li
Date: 2022-07-11 10:19:11
LastEditTime: 2022-09-27 17:11:09
LastEditors: Jiashu Li
Description: 
FilePath: /asr_lm/nn_ss/bin/average_model.py
'''
import os
import sys
import argparse
import glob

import yaml
import numpy as np
import torch

sys.path.append("../asr_lm")

from utils.checkpoint import average_checkpoints


def get_args():
    parser = argparse.ArgumentParser(description='average model')
    parser.add_argument('--dst_model',
                        default="",
                        required=True,
                        help='averaged model')
    parser.add_argument('--src_path',
                        default="",
                        required=True,
                        help='src model path for average')
    parser.add_argument('--checkpoint_prefix',
                        default="checkpoint-",
                        required=True,
                        help='checkpoint prefix for load model')
                        
    parser.add_argument('--val_best',
                        action="store_false",
                        help='averaged model')
    parser.add_argument('--num',
                        default=5,
                        type=int,
                        help='nums for averaged model')
    parser.add_argument('--min_epoch',
                        default=0,
                        type=int,
                        help='min epoch used for averaging model')
    parser.add_argument('--max_epoch',
                        default=65536,
                        type=int,
                        help='max epoch used for averaging model')

    args = parser.parse_args()
    print(args)
    return args


def main():
    args = get_args()
    checkpoint_prefix = args.checkpoint_prefix
    checkpoints = []
    val_scores = []
    if args.val_best:
        yamls = glob.glob('{}/[!train]*.yaml'.format(args.src_path))
        yamls = [_y for _y in yamls if not 'Transformer' in os.path.basename(_y)]
        for y in yamls:
            with open(y, 'r') as f:
                dic_yaml = yaml.load(f, Loader=yaml.FullLoader)
                # print(f)
                # print(dic_yaml)
                loss = dic_yaml['train_loss']
                steps = dic_yaml['local_steps']
                if steps >= args.min_epoch and steps <= args.max_epoch:
                    val_scores += [[steps, loss]]
        val_scores = np.array(val_scores)
        sort_idx = np.argsort(val_scores[:, -1])
        sorted_val_scores = val_scores[sort_idx][::1]
        print("best val scores = " + str(sorted_val_scores[:args.num, 1]))
        print("selected epochs = " +
              str(sorted_val_scores[:args.num, 0].astype(np.int64)))
        path_list = [
            args.src_path + '/{}{}.pt'.format(checkpoint_prefix, int(epoch))
            for epoch in sorted_val_scores[:args.num, 0]
        ]
    else:
        path_list = glob.glob('{}/{}[0-9]*.pt'.format(args.src_path, checkpoint_prefix))
        path_list = sorted(path_list, key=os.path.getmtime)
        path_list = path_list[-args.num:]

    avg = average_checkpoints(path_list)

    print('Saving to {}'.format(args.dst_model))
    torch.save(avg, args.dst_model)


if __name__ == '__main__':
    main()
